/*
** macints.h -- Macintosh Linux interrupt handling structs and prototypes
**
** Copyright 1997 by Michael Schmitz
**
** This file is subject to the terms and conditions of the GNU General Public
** License.  See the file COPYING in the main directory of this archive
** for more details.
**
*/

#ifndef _LINUX_MACINTS_H_
#define _LINUX_MACINTS_H_

#include <asm/irq.h>

/*
** Macintosh Interrupt sources.
**
** Note: these are all routed via the generic VIA interrupt routine!
**
*/

#define SRC_VIA1		0
#define SRC_VIA2		1

#define VIA1_SOURCE_BASE	8
#define VIA2_SOURCE_BASE	16
#define RBV_SOURCE_BASE		24
#define MAC_SCC_SOURCE_BASE	32
#define NUBUS_SOURCE_BASE	56
#define NUBUS_MAX_SOURCES	8

/* FIXME: sources not contigous ... */
#define NUM_MAC_SOURCES   (NUBUS_SOURCE_BASE+NUBUS_MAX_SOURCES-VIA1_SOURCE_BASE)

/* broken logic :-) Bits 0-7 are IRQ index, 8-whatever source */
#define IRQ_SRC_MASK (VIA1_SOURCE_BASE|VIA2_SOURCE_BASE|MAC_SCC_SOURCE_BASE)
#define IRQ_IDX_MASK 7

/* interrupt service types */
#define IRQ_TYPE_SLOW     0
#define IRQ_TYPE_FAST     1
#define IRQ_TYPE_PRIO     2

#define	IRQ_SPURIOUS      (IRQ_MACHSPEC | 0)

/* auto-vector interrupts */
#define IRQ_AUTO_1        (IRQ_MACHSPEC | 1)
#define IRQ_AUTO_2        (IRQ_MACHSPEC | 2)
#define IRQ_AUTO_3        (IRQ_MACHSPEC | 3)
#define IRQ_AUTO_4        (IRQ_MACHSPEC | 4)
#define IRQ_AUTO_5        (IRQ_MACHSPEC | 5)
#define IRQ_AUTO_6        (IRQ_MACHSPEC | 6)
#define IRQ_AUTO_7        (IRQ_MACHSPEC | 7)

/* Level 1 (VIA1) interrupts */
#define IRQ_VIA1_0	  (IRQ_MACHSPEC | 8)	/* one second int. */
#define IRQ_VIA1_1        (IRQ_MACHSPEC | 9)	/* VBlank int. */
#define IRQ_MAC_VBL	  IRQ_VIA1_1
#define IRQ_VIA1_2 	  (IRQ_MACHSPEC | 10)	/* ADB SR shifts complete */
#define IRQ_MAC_ADB	  IRQ_VIA1_2
#define IRQ_MAC_ADB_SR	  IRQ_VIA1_2
#define IRQ_VIA1_3	  (IRQ_MACHSPEC | 11)	/* ADB SR CB2 ?? */
#define IRQ_MAC_ADB_SD	  IRQ_VIA1_3
#define IRQ_VIA1_4        (IRQ_MACHSPEC | 12)	/* ADB SR ext. clock pulse */
#define IRQ_MAC_ADB_CL	  IRQ_VIA1_4
#define IRQ_VIA1_5	  (IRQ_MACHSPEC | 13)
#define IRQ_MAC_TIMER_2	  IRQ_VIA1_5
#define IRQ_VIA1_6	  (IRQ_MACHSPEC | 14)
#define IRQ_MAC_TIMER_1	  IRQ_VIA1_6
#define IRQ_VIA1_7        (IRQ_MACHSPEC | 15)

/* Level 2 (VIA2/RBV/OSS) interrupts */
#define IRQ_VIA2_0	  (IRQ_MACHSPEC | 16)
#define IRQ_MAC_SCSIDRQ	  IRQ_VIA2_0
#define IRQ_VIA2_1        (IRQ_MACHSPEC | 17)
#define IRQ_MAC_NUBUS	  IRQ_VIA2_1
#define IRQ_VIA2_2 	  (IRQ_MACHSPEC | 18)
#define IRQ_VIA2_3	  (IRQ_MACHSPEC | 19)
#define IRQ_MAC_SCSI	  IRQ_VIA2_3
#define IRQ_VIA2_4        (IRQ_MACHSPEC | 20)
#define IRQ_VIA2_5	  (IRQ_MACHSPEC | 21)
#define IRQ_VIA2_6	  (IRQ_MACHSPEC | 22)
#define IRQ_VIA2_7        (IRQ_MACHSPEC | 23)

#if 0
/* RBV interrupts */
#define IRQ_RBV_0	  (IRQ_MACHSPEC | 24)
#define IRQ_RBV_1	  (IRQ_MACHSPEC | 25)
#define IRQ_RBV_2 	  (IRQ_MACHSPEC | 26)
#define IRQ_RBV_3	  (IRQ_MACHSPEC | 27)
#define IRQ_RBV_4	  (IRQ_MACHSPEC | 28)
#define IRQ_RBV_5	  (IRQ_MACHSPEC | 29)
#define IRQ_RBV_6	  (IRQ_MACHSPEC | 30)
#define IRQ_RBV_7	  (IRQ_MACHSPEC | 31)
#endif

/* Level 3 (PSC, AV Macs only) interrupts */
#define IRQ_PSC3_0	  (IRQ_MACHSPEC | 24)
#define IRQ_MAC_MACE	  IRQ_PSC3_0
#define IRQ_PSC3_1	  (IRQ_MACHSPEC | 25)
#define IRQ_PSC3_2	  (IRQ_MACHSPEC | 26)
#define IRQ_PSC3_3	  (IRQ_MACHSPEC | 27)

/* Level 4 (SCC) interrupts */
#define IRQ_SCC 	     (IRQ_MACHSPEC | 32)
#define IRQ_SCCB	     (IRQ_MACHSPEC | 33)
#define IRQ_SCCA	     (IRQ_MACHSPEC | 34)
#if 0 /* FIXME: are there multiple interrupt conditions on the SCC ?? */
/* SCC interrupts, non autovector */
#define IRQ_SCCB_TX	     (IRQ_MACHSPEC | 32)
#define IRQ_SCCB_STAT	     (IRQ_MACHSPEC | 33)
#define IRQ_SCCB_RX	     (IRQ_MACHSPEC | 34)
#define IRQ_SCCB_SPCOND	     (IRQ_MACHSPEC | 35)
#define IRQ_SCCA_TX	     (IRQ_MACHSPEC | 36)
#define IRQ_SCCA_STAT	     (IRQ_MACHSPEC | 37)
#define IRQ_SCCA_RX	     (IRQ_MACHSPEC | 38)
#define IRQ_SCCA_SPCOND	     (IRQ_MACHSPEC | 39)
#endif

/* Level 4 (PSC, AV Macs only) interrupts */
#define IRQ_PSC4_0	  (IRQ_MACHSPEC | 32)
#define IRQ_PSC4_1	  (IRQ_MACHSPEC | 33)
#define IRQ_PSC4_2	  (IRQ_MACHSPEC | 34)
#define IRQ_PSC4_3	  (IRQ_MACHSPEC | 35)
#define IRQ_MAC_MACE_DMA  IRQ_PSC4_3

/* Level 5 (PSC, AV Macs only) interrupts */
#define IRQ_PSC5_0	  (IRQ_MACHSPEC | 40)
#define IRQ_PSC5_1	  (IRQ_MACHSPEC | 41)
#define IRQ_PSC5_2	  (IRQ_MACHSPEC | 42)
#define IRQ_PSC5_3	  (IRQ_MACHSPEC | 43)

/* Level 6 (PSC, AV Macs only) interrupts */
#define IRQ_PSC6_0	  (IRQ_MACHSPEC | 48)
#define IRQ_PSC6_1	  (IRQ_MACHSPEC | 49)
#define IRQ_PSC6_2	  (IRQ_MACHSPEC | 50)
#define IRQ_PSC6_3	  (IRQ_MACHSPEC | 51)

/* Nubus interrupts (cascaded to VIA2) */
#define IRQ_NUBUS_1	     (IRQ_MACHSPEC | 56)

#define INT_CLK   24576	    /* CLK while int_clk =2.456MHz and divide = 100 */
#define INT_TICKS 246	    /* to make sched_time = 99.902... HZ */


#define VIA_ENABLE	0
#define VIA_PENDING	1
#define VIA_SERVICE	2
#define VIA_MASK	3

/* 
 * Utility functions for setting/clearing bits in the interrupt registers of
 * the VIA. 
 */

void mac_enable_irq( unsigned irq );
void mac_disable_irq( unsigned irq );
void mac_turnon_irq( unsigned irq );
void mac_turnoff_irq( unsigned irq );
void mac_clear_pending_irq( unsigned irq );
int  mac_irq_pending( unsigned irq );
int  nubus_request_irq(int slot, void (*handler)(int,void *,struct pt_regs *));
int  nubus_free_irq(int slot);

unsigned long mac_register_nubus_int( void );
void mac_unregister_nubus_int( unsigned long );

extern void mac_default_handler(int irq, void *dev_id, struct pt_regs *regs);
extern void via1_irq(int irq, void *dev_id, struct pt_regs *regs);
extern void via2_irq(int irq, void *dev_id, struct pt_regs *regs);
extern void  rbv_irq(int irq, void *dev_id, struct pt_regs *regs);
extern void mac_bang(int irq, void *dev_id, struct pt_regs *regs);
		
extern void mac_SCC_handler(int irq, void *dev_id, struct pt_regs *regs);

#endif /* linux/macints.h */
